@@ -2,8 +2,10 @@
 # coding: utf-8
 
 import collections
+import glob
 import logging
 import os
+import pathlib
 import re
 import sys
 import tempfile
@@ -17,61 +19,12 @@ from .plugin import PixiePluginContext
 from .plugins import load_plugins
 from .rendering import render_options, render_text, render_value
 from .runtime import PixieRuntime
+from . import utils
 
 
 _log = logging.getLogger(__name__)
 
 
-# def complete(text, state):
-#     if str(text).startswith('~/'):
-#         home = os.path.expanduser('~/')
-#         p = os.path.join(home, text[2:])
-#     else:
-#         p = text
-#         home = None
-
-#     items = pathlib.Path(os.getcwd()).glob(p + '*')
-#     if items is not None and home is not None:
-#         items = ['~/' + x[len(home):] for x in items]
-#     return (items + [None])[state]
-
-
-# def set_readline():
-#     try:
-#         import readline
-#         readline.set_completer_delims(' \t\n;')
-#         readline.parse_and_bind("tab: complete")
-#         readline.set_completer(complete)
-#     except:
-#         pass
-
-
-# class AttributeDict(dict):
-#     __getattr__ = dict.__getitem__
-#     __setattr__ = dict.__setitem__
-
-
-# color = AttributeDict({
-#     'PURPLE': '\033[35m',
-#     'CYAN':  '\033[36m',
-#     'BLUE':  '\033[34m',
-#     'GREEN':  '\033[32m',
-#     'YELLOW':  '\033[33m',
-#     'RED':  '\033[31m',
-#     'BOLD':  '\033[1m',
-#     'UNDERLINE':  '\033[4m',
-#     'ITALIC':  '\033[3m',
-#     'END':  '\033[0m',
-# })
-
-
-# def dict_to_str(d, fmt='%s=%s\n'):
-#     s = ''
-#     for x in d:
-#         s += fmt % (x, d[x])
-#     return s
-
-
 def str2bool(v):
     if v is None:
         return False
@@ -86,38 +39,6 @@ known_types = {
 }
 
 
-# def term_color(text, *text_colors):
-#     return ''.join(text_colors) + text + color.END
-
-
-
-
-# def render_file(path, context):
-#     """Used to render a Jinja template."""
-
-#     template_dir, template_name = os.path.split(path)
-#     return render(template_name, context, template_dir)
-
-
-# def is_enabled(options):
-#     if 'enabled' in options:
-#         return options['enabled']
-#     if 'disabled' in options:
-#         return not options['disabled']
-#     if 'enabledif' in options:
-#         enabledif = options['enabledif']
-#         value = enabledif['value']
-#         if 'equals' in enabledif:
-#             return value == enabledif['equals']
-#         elif 'notequals' in enabledif:
-#             return value != enabledif['notequals']
-#     return True
-
-
-# def read_input(s):
-#     return input(s)
-
-
 def convert(v, type):
     if type in known_types:
         return known_types[type](v)
@@ -139,7 +60,7 @@ def read_parameter(prompt, context, runtime: PixieRuntime):
 
 def config_cli(args):
     options = {}
-    scaffold_file = os.path.expanduser('~/.xscaffold')
+    scaffold_file = os.path.expanduser('~/.pixie')
 
     yaml = YAML()
     if os.path.exists(scaffold_file):
@@ -166,7 +87,6 @@ def rm_rf(d):
 
 def locate_scaffold_file(path, name):
     base_paths = [
-        './',
         path
     ]
 
@@ -179,26 +99,29 @@ def locate_scaffold_file(path, name):
 
     names = [
         f'{name}',
-        f'.{name}'
+        f'.{name}',
+        os.path.join(name, '.pixie')
     ]
     for base_path in base_paths:
         for ext in extensions:
             for n in names:
                 full_path = os.path.join(base_path, n + ext)
-                _log.debug(f'locating pix script using {full_path}')
+                _log.debug(f'locating pixie script using {full_path}')
                 if os.path.exists(full_path) and os.path.isfile(full_path):
                     return full_path
     return None
 
 
 def process_parameters(parameters, context: PixieContext, runtime: PixieRuntime):
-    for parameter in parameters:
+    for parameter in (parameters or []):
         parameter_options = render_options(parameter, context)
         parameter_name = parameter_options['name']
-        if parameter_name in context:
-            context[parameter_name] = convert(context[parameter_name], parameter.get('type', 'str'))
+        context_source = parameter_options.get('context_source', parameter_name)
+        context_target = parameter_options.get('context_target', parameter_name)
+        if context_source in context:
+            context[context_target] = convert(context[context_source], parameter.get('type', 'str'))
         else:
-            context[parameter_name] = read_parameter(parameter_options, context, runtime)
+            context[context_target] = read_parameter(parameter_options, context, runtime)
 
 
 def run(context: PixieContext, options, runtime: PixieRuntime):
@@ -211,8 +134,6 @@ def run(context: PixieContext, options, runtime: PixieRuntime):
 
 
 def execute_scaffold(context: PixieContext, options, runtime: PixieRuntime):
-    packages_dir = os.path.realpath(os.path.expanduser('~/.pixie/packages'))
-    tempdir = options.get('temp', packages_dir)
     package = options['package']
 
     yaml = YAML()
@@ -227,10 +148,10 @@ def execute_scaffold(context: PixieContext, options, runtime: PixieRuntime):
         _log.debug('using local package \'%s\'', package_path)
         pkg_dir = package_path
     else:
-        pkg_dir = fetch_git(runtime, tempdir, package)
+        pkg_dir = fetch_package(runtime, options, package)
     
     scaffold_file = locate_scaffold_file(pkg_dir, script)
-    _log.debug('scaffold file: %s', scaffold_file)
+    _log.debug('using pixie file: %s', scaffold_file)
 
     if scaffold_file:
         pkg_dir = os.path.dirname(scaffold_file)
@@ -244,7 +165,12 @@ def execute_scaffold(context: PixieContext, options, runtime: PixieRuntime):
         config = {
             'jobs': {
                 job_name: {
-                    'steps': options.get('steps', [{ 'action': 'fetch' }])
+                    'steps': options.get('steps', [{
+                        'action': 'fetch',
+                        'with': {
+                            'source': '${{ source | default(".") }}'
+                        }
+                    }])
                 }
             }
         }
@@ -283,6 +209,12 @@ def execute_scaffold(context: PixieContext, options, runtime: PixieRuntime):
     context_options = render_options(job.get('context', {}), context)
     context.update(context_options)
     process_parameters(job.get('parameters', []), context, runtime)
+
+    if context.get('__target') is None:
+        context['__target'] = '.'
+    
+    _log.debug('%s', context)
+
     steps: list = job.get('steps', [])
 
     step_execution.execute(context, runtime, steps_context, steps)
@@ -290,7 +222,31 @@ def execute_scaffold(context: PixieContext, options, runtime: PixieRuntime):
     return context
 
 
-def fetch_git(runtime, tempdir, package):
+def discover(runtime, options, package):
+    result = {}
+    pkg_dir = pathlib.Path(fetch_package(runtime, options, package))
+    for f in pkg_dir.glob('**/.pixie.yaml'):
+        pixie_config = utils.read_yaml(str(f), {})
+        if 'name' in pixie_config:
+            pkg_name = pixie_config['name']
+            jobs = pixie_config.get('jobs')
+            for job_name in jobs:
+                job = jobs[job_name]
+                alias_name = f'{pkg_name}/{job_name}'
+                result[alias_name] = dict(
+                    package=package,
+                    job=job_name,
+                    description=job.get('description', ''),
+                    script=str(f.relative_to(pkg_dir))
+                )
+    return result
+
+
+def fetch_package(runtime, options, package):
+
+    packages_dir = os.path.realpath(os.path.expanduser('~/.pixie/packages'))
+    tempdir = options.get('temp', packages_dir)
+
     package_parts = package.split('@')
     if len(package_parts) == 1:
         package_name = package_parts[0]
